
var Cache = function(config) {

    var self = this;
    self.__cache__ = {};
    self.__length__ = 0;
    self.__head__ = null;
    self.__tail__ = null;

    // 记录当前进程的初始化内存大小
    self.__startMemoryInfo__ = process.memoryUsage();

    config = config || {};

    self.max = config.max || 10 * 1024 * 1024;
    self.defaultexpire = config.defaultexpire || 60 * 60 * 1000;
    config.update && self.update();

};

Cache.prototype.set = function(key, value) {

    var self = this;
    var __cache__ = self.__cache__;
    var record = __cache__[key];

    if (!record) {

        record = {
            key: key,
            value: value,
            deadline: Date.now() + self.defaultExpire
        };
        

    } else {

        record.deadline = Date.now() + self.defaultExpire;
        record.value = value;
        this.remove(key);

    }

    this.unshift(record);

};

Cache.prototype.get = function(key) {
    var value;
    var record = this.__cache__[key];

    if (record) {
        var now = Date.now();
        if (record.deadline < now) {
            this.remove(key);
        } else {
            record.deadline = Date.now() + this.defaultExpire;
            value = record.value;
            this.remove(key);
            this.__cache__[key] = record;
            this.unshift(record);            
        }
    }
    return value;

};

Cache.prototype.remove = function(key) {
    
    var __cache__ = this.__cache__;
    var record = __cache__[key];
    
    if (!record) return;

    __cache__[key] = undefined;
    this.__length__ -= 1;

    var recNext = record.next;
    var recPrev = record.prev;

    if (recNext && recPrev) {
        recNext.prev = recPrev;
        recPrev.next = recNext;
        return;
    }

    if (recNext) {
        this.__head__ = recNext;
        recNext.prev = null;
        return;
    }

    if (recPrev) {
        this.__tail__ = recPrev;
        recPrev.next = null;
        return;
    }

    this.__head__ = this.__tail__ = null;

};

Cache.prototype.push = function(record) {

    this.__length__ += 1;
    
    if (!this.__tail__) {
        this.__head__ = this.__tail__ = record;
        return;
    }

    this.__tail__.next = record;
    record.prev = this.__tail__;
    record.next = null;
    this.__tail__ = record;

};

Cache.prototype.pop = function() {

    if (!this.__tail__) return;
    
    var record = this.__tail__;

    if (record.prev) {
        record.prev.next = null;
        this.__tail__ = record.prev;
    } else {
        this.__tail__ = this.__head__ = null;
    }

    this.__length__ -= 1;
    this.__cache__[record.key] = null;
    return record;

};

Cache.prototype.unshift = function(record) {
    this.__cache__[record.key] = record;
    this.__length__ += 1;
    if (!this.__head__) {
        this.__head__ = this.__tail__ = record;
        return;
    }
    this.__head__.prev = record;
    record.next = this.__head__;
    record.prev = null;
    this.__head__ = record;    
};

Cache.prototype.shift = function() {

    if (!this.__head__) return;

    var record = this.__head__;

    if (record.next) {
        record.next.prev = null;
        this.__head__ = record.next;
    } else {
        this.__head__ = this.__tail__ = null;
    }

    this.__tail__ -= 1;
    this.__cache__[record.key] = null;
    return record;

};

Cache.prototype.update = function(time) {

    time = time || 5 * 60 * 1000;
    var self = this;
    self.__updateTimer__ = setTimeout(function() {
        var now = self.lastUpdate = Date.now();
        while (self.__tail__ && self.__tail__.deadline < now) {
            self.pop();
        }
        self.update(time);
    }, time);

};

Cache.prototype.print = function() {
    var o = this.__head__;
    while (o) {
        console.log(o.key + ' : ' + o.value + ' | ' + o.deadline);
        o = o.next;
    }
};

exports.init = function(config) {
    return new Cache(config);
};

